@@ -16,6 +16,8 @@ class Iterator(object):
 
     fields --
         has_more -- boolean -- True if there are more results in the db, false otherwise
+        ifilter -- callback -- an iterator filter, all yielded rows will be passed
+            through this callback and skipped if ifilter(row) returns True
 
     examples --
         # iterate through all the primary keys of some orm
@@ -34,14 +36,22 @@ class Iterator(object):
         query -- Query -- the query instance that produced this iterator
         """
         self.results = results
+        self.ifilter = None
         self.orm = orm
         self.has_more = has_more
-        self.query = copy.deepcopy(query)
+        self.query = query.copy()
         self._values = False
         self.reset()
 
     def reset(self):
-        self.iresults = (self._get_result(x) for x in self.results)
+        #self.iresults = (self._get_result(d) for d in self.results)
+        #self.iresults = self.create_generator()
+        inormalize = (self._get_result(d) for d in self.results)
+        self.iresults = (o for o in inormalize if not self._filtered(o))
+
+    def _filtered(self, o):
+        """run orm o through the filter, if True then orm o should not be included"""
+        return self.ifilter(o) if self.ifilter else False
 
     def next(self):
         return self.iresults.next()
@@ -110,6 +120,11 @@ class Iterator(object):
         field_name = self.orm.schema.field_name(k)
         return (getattr(r, field_name, None) for r in self)
 
+    def create_generator(self):
+        """put all the pieces together to build a generator of the results"""
+        inormalize = (self._get_result(d) for d in self.results)
+        return (o for o in inormalize if not self._filtered(o))
+
     def _get_result(self, d):
         r = None
         if self._values:
@@ -131,6 +146,8 @@ class AllIterator(Iterator):
     chunk of results until there are no more results of the passed in Query(), so you
     can just iterate through every row of the db without worrying about pulling too
     many rows at one time
+
+    NOTE -- pop() may have unexpected results
     """
     def __init__(self, query):
         limit, offset, _ = query.get_bounds()
@@ -153,7 +170,7 @@ class AllIterator(Iterator):
 
         else:
             # k is not in here, so let's just grab it
-            q = copy.deepcopy(self.query)
+            q = self.query.copy()
             orm = q.set_offset(k).get_one()
             if orm:
                 v = self._get_result(orm.fields)
@@ -162,11 +179,11 @@ class AllIterator(Iterator):
 
         return v
 
-    def __len__(self):
+    def count(self):
         ret = 0
         if self.results.has_more:
             # we need to do a count query
-            q = copy.deepcopy(self.query)
+            q = self.query.copy()
             q.set_limit(0).set_offset(0)
             ret = q.count()
         else:
@@ -185,6 +202,9 @@ class AllIterator(Iterator):
             self.offset += self.chunk_limit
             self._set_results()
 
+#    def next(self):
+#        raise NotImplementedError()
+
     def _set_results(self):
         self.results = self.query.set_offset(self.offset).get(self.chunk_limit)
         if self._values:
@@ -209,6 +229,7 @@ class AllIterator(Iterator):
         self.results = self.results.values()
         return super(AllIterator, self).values()
 
+
 class Query(object):
     """
     Handle standard query creation and allow interface querying
@@ -269,7 +290,12 @@ class Query(object):
         return q
 
     def __iter__(self):
-        return self.get()
+        #return self.all()
+        #return self.get()
+        # NOTE -- for some reason I need to call AllIterator.__iter__() explicitely
+        # because it would call AllIterator.next() even though AllIterator.__iter__
+        # returns a generator, not sure what's up
+        return self.get() if self.has_limit() else self.all().__iter__()
 
     def copy(self):
         """nice handy wrapper around the deepcopy"""
